% Clermon Auvergne University
% T. Chateau
% track.m
function track_harris
Basename = 'im/';   % basename of image files
ext = 'ppm' ; % extension used
FirstImage = 10 ; % first image of the sequence
LastImage = 100 ; % last  image of the sequence
step = 1 ; % step between 2 images
first = 1 ;
Pmem = [];

for i=FirstImage:step:LastImage
    filename = sprintf('%s%0.8d.%s',Basename,i,ext);
    imb=imread(filename);
    imb=imb(:,:,1); % we keep the red component
    im=double(imb);
    figure (1) ; subplot(1,2,1); imshow(imb);
    drawnow;
    title(filename);
    if(first)
        imod = im;
        figure (1) ; subplot(1,2,1); imshow(imb);
        drawnow;
        [x,y]=ginput(1)
        %% initialisation
        first=0;
        motion=[0,0];
        %extract model 'interest points'
        [px,py]=harris_func(im)
        mod = [px py];
        % compute distance to x,y (point to track)
        Pt = [x,y];
        Pt = repmat(Pt,size(mod,1),1);
        dist = (mod-Pt)*(mod-Pt)';
        dist = diag(dist);
        dist = sqrt(dist);
        th = 50 ;% pixels
        % Keep points with distance up to th. 
        [ind,val]=find(dist<th);
        Ptmod = mod(ind,:);
        % compute descriptors (ng in a ROI of size 11x11 (5x2+1)
        zmod=descriptor(im,Ptmod,5);
        pest = [x,y];
    else
        %extract interest points of the current image
        [px,py]=harris_func(im)
        Ptcur = [px py];
        % compute distance to previous position (point to track)
        Pt = pest;
        Pt = repmat(Pt,size(Ptcur,1),1);
        dist = (Ptcur-Pt)*(Ptcur-Pt)';
        dist = diag(dist);
        dist = sqrt(dist);
        th = 50 ;% pixels
        % Keep points with distance up to th. 
        [ind,val]=find(dist<th);
        Ptcur = Ptcur(ind,:);
        % compute descriptors (ng in a ROI of size 11x11 (5x2+1)
        sroi = 5;
        zcur=descriptor(im,Ptcur,sroi);
        % compute descriptor interdistance matrix
        Dz = distSqrd(zmod,zcur);
        % compute spatial interdistance matrix
        Ds = distSqrd(Ptmod+repmat(motion,size(Ptmod,1),1),Ptcur);
        % Keep distances up to thdist
        thdist = 15; % pixels
        MaskDs = Ds < thdist^2;
        % Apply mask to Dz (set large distance to distant points
        Dz2 = MaskDs.*Dz+ (1-MaskDs).*1000000;
        % Match points using the Nearest Neighbour recursive function
        th = 30 ; % th in ng
        th2 = (th^2*((2*sroi+1)^2));
        indl = []; indc = [];
        [indl, indc] = match(Dz2,th2,indl,indc)
        Pts1 = Ptmod(indl,:);
        Pts2 = Ptcur(indc,:);
        % Compute median motion
        Ptsmotion = Pts2 - Pts1;
        motion = median(Ptsmotion);
        %% update estimation x and y
        pest = [x,y]+ motion;
        % Update position
        Pmem = [Pmem ; pest]; % Save pest
        l = pest(2);
        c = pest(1);
        %% Display
        figure (1) ; subplot(1,2,1); imshow(imb);
        drawnow;
        subplot(1,2,2)
        I = [imod zeros(size(imod,1),size(im,2)) ; zeros(size(im,1),size(imod,2))  im];
        image(I);
        colormap(gray(256));
        hold on 
        plot(Pts1(:,1),Pts1(:,2),'+r');
        plot(size(imod,2)+Pts2(:,1),size(imod,1)+Pts2(:,2),'+r');
        title('Images and matches')
        % Green Points denote matched points between reference image and pattern
        for n=1:size(Pts1,1)
            plot([Pts1(n,1);size(imod,2)+Pts2(n,1)],[Pts1(n,2);size(imod,1)+Pts2(n,2)]);
        end
        hold off
        subplot(1,2,1)
        hold on
        plot(c,l,'*r');
        plot(Pmem(:,1),Pmem(:,2),'-r');
        %plot(Ptmod(:,1),Ptmod(:,2),'+g');
        hold off
    end
    drawnow;
end

function [vote] = calc_poids(m,sigma,N);
% m : foreground binary image
% sigma : bandwidth
% number of points drawn (subsampling)
vote = zeros(size(m,1),size(m,2));
[i,j]=find(m(:,:)==0);%foreground
if(~isempty(i))
    if(size(i,1)>N)
        c=randperm(size(i,1));
    else
        c=1:size(i,1);
        N = size(i,1);
    end
    i=i(c(1:N));
    j=j(c(1:N));
    alpha=1;
    for k1=1:length(i)
        for k2=1:length(i)
            cout=exp(-alpha*(((i(k1)-i(k2))^2 + (j(k1)-j(k2))^2))/(2*sigma^2))/(((2*pi)^0.5)*sigma);
            vote(i(k1),j(k1))=vote(i(k1),j(k1))+cout;
        end
    end
end


%% Return the (2n+1)x(2n+1) gray level ROIs centered on Pts (Nbx2)
% WARNING : may crash for interest points near to the border of the image (out of bound indexes)
function z=descriptor(Im,Pts,n)
[nl,nc]=size(Pts);
z = zeros(nl,(2*n+1)^2);
for l = 1:nl
     tmp = Im(Pts(l,2)-n:Pts(l,2)+n,Pts(l,1)-n:Pts(l,1)+n);
     z(l,:) = [tmp(:)]';
end

%% Compute interdistance matrix between X and Y (x1^t,x2^t,x3^t,...)^t
function D2 = distSqrd(X,Y)
    nx      = size(X,1);
    ny      = size(Y,1);
    D2      = sum(X.^2,2)*ones(1,ny) + ones(nx,1)*sum(Y.^2,2)' - 2*(X*Y');
    
    
%% Matching
% This is a recursive function
% It should be initialize with empty variables indl and indc
% It returns line and coloumn vector indexes corresponding to matched features
function [indl, indc] = match(M1,th,indl,indc)
% brut force matching 
% Min: input distance matrix
% th: distance threshold
% indl: selected lines (initialized to null vector)
% indc: selected columns (initialized to null vector)
[val,ind]=min(M1);
[va2,indctmp]=min(val);
if va2<th
    indl = [ indl ind(indctmp)];
    indc = [ indc indctmp];
    M1(ind(indc),:)=th+1;
    M1(:,indctmp)=th+1;    
    [indl, indc] = match(M1,th,indl,indc);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

